'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Collapse = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactMotion = require('react-motion');

var _ReactComponentWithPureRenderMixin = require('react/lib/ReactComponentWithPureRenderMixin');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var SPRING_PRECISION = 1;

var WAITING = 'WAITING';
var RESIZING = 'RESIZING';
var RESTING = 'RESTING';
var IDLING = 'IDLING';

var noop = function noop() {
  return null;
};
var css = {
  collapse: 'ReactCollapse--collapse',
  content: 'ReactCollapse--content'
};

var Collapse = (0, _createReactClass2.default)({
  propTypes: {
    isOpened: _propTypes2.default.bool.isRequired,
    springConfig: _propTypes2.default.objectOf(_propTypes2.default.number),
    forceInitialAnimation: _propTypes2.default.bool,

    hasNestedCollapse: _propTypes2.default.bool,

    fixedHeight: _propTypes2.default.number,

    theme: _propTypes2.default.objectOf(_propTypes2.default.string),
    style: _propTypes2.default.object,

    onRender: _propTypes2.default.func,
    onRest: _propTypes2.default.func,
    onMeasure: _propTypes2.default.func,

    children: _propTypes2.default.node.isRequired
  },

  getDefaultProps: function getDefaultProps() {
    return {
      forceInitialAnimation: false,
      hasNestedCollapse: false,
      fixedHeight: -1,
      style: {},
      theme: css,
      onRender: noop,
      onRest: noop,
      onMeasure: noop
    };
  },
  getInitialState: function getInitialState() {
    return {
      currentState: IDLING,
      from: 0,
      to: 0
    };
  },
  componentDidMount: function componentDidMount() {
    var _props = this.props,
        isOpened = _props.isOpened,
        forceInitialAnimation = _props.forceInitialAnimation,
        onRest = _props.onRest;

    if (isOpened) {
      var to = this.getTo();
      if (forceInitialAnimation) {
        var from = this.wrapper.clientHeight;
        this.setState({ currentState: RESIZING, from: from, to: to });
      } else {
        this.setState({ currentState: IDLING, from: to, to: to });
      }
    }
    onRest();
  },
  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    if (nextProps.hasNestedCollapse) {
      // For nested collapses we do not need to change to waiting state
      // and should keep `height:auto`
      // Because children will be animated and height will not jump anyway
      // See https://github.com/nkbt/react-collapse/issues/76 for more details
      if (nextProps.isOpened !== this.props.isOpened) {
        // Still go to WAITING state if own isOpened was changed
        this.setState({ currentState: WAITING });
      }
    } else if (this.state.currentState === IDLING && (nextProps.isOpened || this.props.isOpened)) {
      this.setState({ currentState: WAITING });
    }
  },


  shouldComponentUpdate: _ReactComponentWithPureRenderMixin.shouldComponentUpdate,

  componentDidUpdate: function componentDidUpdate(_, prevState) {
    var _props2 = this.props,
        isOpened = _props2.isOpened,
        onRest = _props2.onRest,
        onMeasure = _props2.onMeasure;


    if (this.state.currentState === IDLING) {
      onRest();
      return;
    }

    if (prevState.to !== this.state.to) {
      onMeasure({ height: this.state.to, width: this.content.clientWidth });
    }

    var from = this.wrapper.clientHeight;
    var to = isOpened ? this.getTo() : 0;

    if (from !== to) {
      this.setState({ currentState: RESIZING, from: from, to: to });
      return;
    }

    if (this.state.currentState === RESTING) {
      this.setState({ currentState: IDLING, from: from, to: to });
    }
  },
  componentWillUnmount: function componentWillUnmount() {
    cancelAnimationFrame(this.raf);
  },
  onContentRef: function onContentRef(content) {
    this.content = content;
  },
  onWrapperRef: function onWrapperRef(wrapper) {
    this.wrapper = wrapper;
  },
  onRest: function onRest() {
    this.raf = requestAnimationFrame(this.setResting);
  },
  setResting: function setResting() {
    this.setState({ currentState: RESTING });
  },
  getTo: function getTo() {
    var fixedHeight = this.props.fixedHeight;

    return fixedHeight > -1 ? fixedHeight : this.content.clientHeight;
  },
  getWrapperStyle: function getWrapperStyle(height) {
    if (this.state.currentState === IDLING && this.state.to) {
      var fixedHeight = this.props.fixedHeight;

      if (fixedHeight > -1) {
        return { overflow: 'hidden', height: fixedHeight };
      }
      return { height: 'auto' };
    }

    if (this.state.currentState === WAITING && !this.state.to) {
      return { overflow: 'hidden', height: 0 };
    }

    return { overflow: 'hidden', height: Math.max(0, height) };
  },
  getMotionProps: function getMotionProps() {
    var springConfig = this.props.springConfig;


    return this.state.currentState === IDLING ? {
      // When completely stable, instantly jump to the position
      defaultStyle: { height: this.state.to },
      style: { height: this.state.to }
    } : {
      // Otherwise, animate
      defaultStyle: { height: this.state.from },
      style: { height: (0, _reactMotion.spring)(this.state.to, _extends({ precision: SPRING_PRECISION }, springConfig)) }
    };
  },
  renderContent: function renderContent(_ref) {
    var height = _ref.height;

    // eslint-disable-line
    var _props3 = this.props,
        _isOpened = _props3.isOpened,
        _springConfig = _props3.springConfig,
        _forceInitialAnimation = _props3.forceInitialAnimation,
        _hasNestedCollapse = _props3.hasNestedCollapse,
        _fixedHeight = _props3.fixedHeight,
        theme = _props3.theme,
        style = _props3.style,
        onRender = _props3.onRender,
        _onRest = _props3.onRest,
        _onMeasure = _props3.onMeasure,
        children = _props3.children,
        props = _objectWithoutProperties(_props3, ['isOpened', 'springConfig', 'forceInitialAnimation', 'hasNestedCollapse', 'fixedHeight', 'theme', 'style', 'onRender', 'onRest', 'onMeasure', 'children']);

    var _state = this.state,
        from = _state.from,
        to = _state.to;

    // DANGEROUS, use with caution, never do setState with it

    onRender({ current: height, from: from, to: to });

    return _react2.default.createElement(
      'div',
      _extends({
        ref: this.onWrapperRef,
        className: theme.collapse,
        style: _extends({}, this.getWrapperStyle(Math.max(0, height)), style)
      }, props),
      _react2.default.createElement(
        'div',
        { ref: this.onContentRef, className: theme.content },
        children
      )
    );
  },
  render: function render() {
    return _react2.default.createElement(_reactMotion.Motion, _extends({}, this.getMotionProps(), {
      onRest: this.onRest,
      children: this.renderContent }));
  }
});
exports.Collapse = Collapse;
//# sourceMappingURL=Collapse.js.map